## Réponses aux questions du TP sur l'ADN
## avril 2019
## équipe enseignante UE info L1 et DIU EIL, Univ Lille

from random import randint

# Qestion 1

def estADN(sequence):
    """
Renvoie une valeur booléenne indiquant si la chaîne de caractères reçue correspond à une séquence ADN.
:param sequence: (string) chaîne de caractères à vérifier
:return: (bool) True si la chaîne de caractères correspond à une séquence ADN, False sinon
        
:CU: Aucune
        
:Exemples :
>>> estADN('ATGCGATC')
True
>>> estADN('ACKT')
False
>>> estADN('ACTK')
False
>>> estADN("")
True
    """
    test = True
    for base in sequence :
        if base != 'A' and base != 'C' and base != 'G' and base != 'T' :
            test = False
    return test

def estADN_v2(sequence) :
    """
Renvoie une valeur booléenne indiquant si la chaîne de caractères reçue correspond à une séquence ADN.
:param sequence: (string) chaîne de caractères à vérifier
:return: (bool) True si la chaîne de caractères correspond à une séquence ADN, False sinon
        
:CU: Aucune
        
:Exemples :
>>> estADN_v2('ATGCGATC')
True
>>> estADN_v2('ACKT')
False
>>> estADN_v2('ACTK')
False
>>> estADN_v2("")
True
    """
    test = True
    i = 0
    while i < len(sequence) and sequence[i] in ('A', 'C', 'G', 'T'):
        i += 1
    return i == len(sequence)

# Question 2

def genereADN(taille):
    """
Renvoie une séquence aléatoire ayant taille bases
:param taille: (int) la taille de la séquence à générer
:return: (string) une séquence ADN aléatoire

:CU: Aucune

:Exemples:
>>> seq = genereADN(10)
>>> len(seq) == 10
True
>>> estADN(seq)
True
    """
    seq = ''
    bases = ('A', 'T', 'G', 'C')
    for i in range(taille):
        seq += bases[randint(0, 3)]
    return seq


# Question 3

def baseComplementaire(base, molecule):
    """
Renvoie la base ADN complémentaire de celle reçue en paramètre
:param base: (string) une base ADN
:param molecule: (string) le type de molécule voulu en sortie
:return: (string) la base ADN ou ARN complémentaire
    
:CU: base == 'A' or base == 'C' or base == 'G' or base == 'T'
:CU: molecule == 'ADN' or molecule == 'ARN'
    
:Exemples:
>>> baseComplementaire('A', 'ADN')
'T'
>>> baseComplementaire('A', 'ARN')
'U'
>>> baseComplementaire('T', 'adn')
'A'
>>> baseComplementaire('C', 'ARN')
'G'
>>> baseComplementaire('G', 'ARN')
'C'
>>> baseComplementaire('X', 'ADN')
Traceback (most recent call last):
...
AssertionError:...
>>> baseComplementaire('A', 'prot')
Traceback (most recent call last):
...
AssertionError:...
    """
    assert base in ('A', 'T', 'G', 'C'), "Le 1er paramètre doit être un nucléotide, soit A, T, G ou C"
    assert molecule.upper() == 'ADN' or molecule.upper() == 'ARN', "le 2ème paramètre doit être soit 'ADN', soit 'ARN'"
    
    if base == 'A' and molecule == 'ADN':
        complementaire = 'T'
    elif base == 'A' and molecule == 'ARN':
        complementaire = 'U'
    elif base == 'T':
        complementaire = 'A'
    elif base == 'C':
        complementaire = 'G'
    else:
        complementaire = 'C'
    return complementaire

# Question 4

def transcrit(adn, debut, fin):
    """
Renvoie l'ARN construit à partir de la sous-séquence de adn
comprise entre les positions debut et fin
:param adn: (string) une séquence ADN
:param debut: (int) la position de début de la transcription (la première base est numérotée 1)
:param fin: (int) la position de fin de la transcription

:CU: estADN(adn)
:CU: 0 < debut < fin <= len(adn)

:Exemple:
>>> transcrit('TTCTTCTTCGTACTTTGTGCTGGCCTCCACACGATAATCC', 4, 23)
'AAGAAGCAUGAAACACGACC'
>>> transcrit('TTCTXTC', 4, 23)
Traceback (most recent call last):
...
AssertionError:...
>>> transcrit('TTCTTCTTCGTACTTTGTGCTGGCCTCCACACGATAATCC', 23, 4)
Traceback (most recent call last):
...
AssertionError:...
    """
    assert estADN(adn), "Le paramètre n'est pas une séquence ADN"
    assert debut < 0 or fin >= len(adn) or debut < fin, "Les positions doivent être dans l'ordre croissant et comprises dans la séquence"
    
    arn = ''
    for base in adn[debut-1:fin]:
        arn += baseComplementaire(base, 'ARN')
    return arn


# Question 5

def codeGenetique(codon):
    """
Renvoie l'abréviation de 3 lettres du nom de l'acide aminé
correspondant au codon passé en paramètre, ou * pour les codons Stop
:param codon: (string) un codon
:return: (string) le nom en 3 lettre de l'acide aminé

:CU: len(codon) == 3
:CU: les lettres de codon ne peuvent être que des A, U, G, C

Exemples:
>>> codeGenetique('GCA')
'A'
>>> codeGenetique('UAA')
'*'
>>> codeGenetique('G')
Traceback (most recent call last):
...
AssertionError:...
>>> codeGenetique('GNG')
Traceback (most recent call last):
...
AssertionError:...
    """
    assert len(codon) == 3, "Le codon doit comporter 3 bases"
    verif = tuple(nt in ('A', 'U', 'G', 'C') for nt in codon)
    assert verif == (True, True, True), "Le codon doit être une séquence ARN"
    
    aa = ''
    if codon in ('UUU', 'UUC'):
        aa = 'F'
    elif codon in ('UUA', 'UUG', 'CUU', 'CUC', 'CUA', 'CUG'):
        aa = 'L'
    elif codon in ('AUU', 'AUC', 'AUA'):
        aa = 'I'
    elif codon in ('AUG'):
        aa = 'M'
    elif codon in ('GUU', 'GUC', 'GUA', 'GUG'):
        aa = 'V'
    elif codon in ('UCU', 'UCC', 'UCA', 'UCG', 'AGU', 'AGC'):
        aa = 'S'
    elif codon in ('CCU', 'CCC', 'CCA', 'CCG'):
        aa = 'P'
    elif codon in ('ACU', 'ACC', 'ACA', 'ACG'):
        aa = 'T'
    elif codon in ('GCU', 'GCC', 'GCA', 'GCG'):
        aa = 'A'
    elif codon in ('UAU', 'UAC'):
        aa = 'Y'
    elif codon in ('UAA', 'UAG', 'UGA'):
        aa = '*'
    elif codon in ('CAU', 'CAC'):
        aa = 'H'
    elif codon in ('CAA', 'CAG'):
        aa = 'Q'
    elif codon in ('AAU', 'AAC'):
        aa = 'N'
    elif codon in ('AAA', 'AAG'):
        aa = 'K'
    elif codon in ('GAU', 'GAC'):
        aa = 'D'
    elif codon in ('GAA', 'GAG'):
        aa = 'E'
    elif codon in ('UGU', 'UGC'):
        aa= 'C'
    elif codon in ('UGG'):
        aa = 'W'
    elif codon in ('CGU', 'CGC', 'CGA', 'CGG', 'AGA', 'AGG'):
        aa = 'R'
    elif codon in ('GGU', 'GGC', 'GGA', 'GGG'):
        aa = 'G'
    
    return aa

# Question 6

def traduit(arnm):
    """
Renvoie la séquenced protéique obtenue par la traduction de l'ARN passé
en paramètre.
:param arnm: (string) une séquence ARN
:return: (string) une séquence protéique
:CU: arnm est de l'ARN
:Exemple:
>>> traduit('AUGCGAAGCCGAAAGAACACCGGCUAA')
'MRSRKNTG*'
    """
    prot = ''
    for indice in range(0, len(arnm), 3):
        codon = arnm[indice:indice+3]
        aa = codeGenetique(codon)
        prot += aa
    return prot

# Question 7

def replique(adn):
    """
Renvoie la séquence ADN complémentaire de celle reçue en paramètre
:param base: (string) une séquence ADN
:return: (string) la séquence ADN complémentaire
:CU: estADN(adn) == True
:Exemples:
>>> replique('ACTG')
'CAGT'
>>> replique('XCTG')
Traceback (most recent call last):
...
AssertionError:...        
    """
    assert estADN(adn), "Le paramètre n'est pas une séquence ADN"
    
    compinv = ''
    for base in adn:
        compinv = baseComplementaire(base, 'ADN') + compinv
    return compinv

def verifSaisie(question, lchoix):
    """
Renvoie la valeur saisie après avoir vérifié qu'elle appartenait
bien au tuple des réponses possibles passées en paramètre, redemande
la valeur sinon.
:param question: (string) la question à poser
:param lchoix: (list of strings) liste de choix possibles
:CU: lchoix non vide et contient des chaînes de caractères en majuscules
    """
    while True:
        reponse = input(question).upper()
        try:
            assert reponse in lchoix
            return reponse
        except AssertionError:
            print("La réponse doit être une des valeurs suivantes :")
            for choix in lchoix[:-1]:
                print(choix, end=" ou ")
            print(lchoix[-1])
        


if __name__ == '__main__':
    import doctest
    doctest.testmod(optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS)

    print("Bienvenue dans notre logiciel de taitement de séquences biologiques")    
    typeseq = verifSaisie("Voulez-vous travailler sur une séquence ADN ou ARN ? ", ("ADN", "ARN"))
        
    if typeseq == 'ADN':
        print("Vous avez choisi de travailler avec de l'ADN.")
        rep = verifSaisie("Voulez-vous 1- saisir une séquence ou 2- générer une séquence aléatoire ? ", ("1", "2"))
        if rep == "1":
            adn = input("Veillez saisir votre séquence :")
            while not estADN(adn):
                print("La séquence ne doit contenir que les lettres : A, T, G ou C")
                adn = input("Veillez saisir de nouveau votre séquence :")

        else:
            pb = True
            while pb:
                try:
                    taille = input("Quelle la taille de séquence voulez-vous générer ? ")
                    taille = int(taille)
                    pb = False
                except ValueError:
                    print("veuillez saisir un nombre s'il vous plaît")
            adn = genereADN(taille)
            print("Voici la séquence générée :", adn)


