La course du chicon
===================


**Objectifs**

-   listes
-   tris
-   lecture/écriture de fichiers et fichiers csv


Introduction
------------

Le dimanche 27 octobre 2019 aura lieu à Baisieux, commune du Nord
frontalière de la Belgique, la [course du chicon](http://courirabaisieux.fr/la-course-du-chicon/).

Cette course se décline sur plusieurs distances : 15km, 10km, 5.3km,
2km, 1.5km, 1km, 0.5km. Nous ne nous intéresserons dans ce TP qu\'à la
version 15km.

Vous avez la responsabilité du traitement informatique des données :
gestion des inscriptions, récolte des performances des concurrents,
publication des résultats.

Préparation
-----------


1.  Récupérez l'[archive](mat_course_chicon.zip) et décompressez la.

 Cette archive contient trois dossiers :

 -   [`src/`](./src) contient deux modules  [src/Time.py](src/Time.py) et
     [src/competitor.py](src/competitor.py)
 - `docs/` contient la documentation de ces deux modules, accessible depuis le fichier index.html
 -  [`data/`](./data)  contient deux jeux de données dans quatre fichiers de données : un
     petit jeu  de données pour faire des tests simples
 ([data/small\_inscrits.csv](data/small_inscrits.csv) et
 [data/small\_performances.csv](data/small_performances.csv)), et un gros jeu de données
 ([data/inscrits.csv](data/inscrits.csv) et [data/performances.csv](data/performances.csv)).



2.  Consultez la documentation des deux modules fournis.




**À faire**

Sauf mention du contraire toutes les fonctions que vous réaliserez le
seront dans un fichier nommé `course_chicon.py`.

1.  Créez dans le dossier `src` ce fichier avec son en-tête.

Gestion des inscrits
--------------------

Votre première tâche est de construire la liste des candidats inscrits à
la compétition.

Les données concernant ces candidats se trouvent dans le fichier
`data/inscrits.csv` (ou `data/small_inscrits.csv`) qui
est un fichier au format
[CSV](https://fr.wikipedia.org/wiki/Comma-separated_values),
c\'est-à-dire un fichier texte contenant des données tabulées.

La première ligne de ce fichier est constituée des libellés des données
qui suivent :

``` {.sourceCode .text}
Prénoms;Noms;Sexes;Date naiss.
```

Elle précise donc que chacune des lignes qui suivent contient dans cet
ordre le prénom, le nom, le sexe et la date de naissance du candidat
inscrit, ces informations étant séparées par un point-virgule.

Avec ces données vous allez construire des candidats à l\'aide de la
fonction `competitor.create`. Il vous faudra attribuer à chacun de ces
candidats un numéro de dossard, obtenu par simple incrémentation d\'un
compteur. Vous ne tiendrez pas compte des dates de naissance (rien
n\'est prévu pour cela dans le module `competitor`).

Tous les candidats seront rassemblés dans une liste.


**À faire**

Réalisez une fonction nommée `read_competitors` paramétrée par le nom du
fichier csv contenant les données des inscrits, qui renvoie la liste de
ces inscrits.

*Indication* Pensez à la méthode `split` des chaînes de caractères.

**À faire**

Testez la validité de votre fonction avec le fichier
`data/small_inscrits.csv`. 
Vérifiez en particulier la longueur de la liste obtenue, ainsi que le
contenu de quelques éléments.


Report des performances
-----------------------

La course achevée, votre deuxième tâche consiste à reporter les
performances des candidats dans les fiches de la liste de ces candidats.

### Lecture des performances

Les données concernant les performances se trouvent dans le fichier
`data/performances.csv` (ou
`data/small_performances.csv`) qui est un fichier au format
CSV.

La première ligne de ce fichier est constituée des libellés des données
qui suivent :

``` {.sourceCode .text}
bib_num;hours;minutes;seconds
```

Elle précise donc que chacune des lignes qui suivent contient dans cet
ordre le numéro de dossard, le nombre d\'heures, de minutes et de
secondes du temps de parcours d\'un candidat, ces informations étant
séparées par un point-virgule.

Seuls les candidats ayant effectivement participé et achevé la course
figurent dans ce fichier. 
Avec ces données vous allez construire une liste de nombres entiers `(bib_num,h,m,s)`.


**À faire**

Réalisez une fonction nommée `read_performances` paramétrée par le nom
du fichier csv contenant les données des performances, qui renvoie la
liste des quadruplets `(bib_num,h,m,s)` contenus dans ce fichier.

**À faire**

Testez la validité de votre fonction avec le fichier
`data/small_performances.csv`.
Vérifiez en particulier la longueur de la liste obtenue, ainsi que le
contenu de quelques éléments.


### Report

Maintenant que vous disposez de la liste des candidats et celle de leurs
performances, votre travail consiste à reporter ces performances dans
les fiches de ces candidats.


**À faire**

Réalisez une fonction nommée `set_performances` paramétrée par les deux
listes qui modifie les fiches des candidats en reportant leur
performance. Cette fonction ne renvoie pas de valeur.

**À faire**

Testez la validité de votre fonction avec les listes produites par le
petit jeu de données.

Trions nos listes
-----------------

Vous allez avoir l'occasion de réutiliser les tris que vous avez
réalisés dans le cadre des activités du bloc 2 (il est aussi possible
d'utiliser la fonction prédéfinie
[`list.sort`](https://docs.python.org/3.7/library/stdtypes.html#list.sort),
mais c'est un peu dommage de ne pas en profiter ici pour réutiliser
votre code).

Récupérez donc le fichier contenant la fonction de tri que vous
aurez choisie parmi celles réalisées et importez cette fonction.


**À faire**

Sur le modèle des fonctions de comparaison que vous avez déjà
rencontrées, réalisez dans le module `course_chicon`, une fonction
nommée `compare_name` qui compare selon l'ordre alphabétique des noms
des deux compétiteurs passés en paramètre.

Utilisez cette fonction pour trier dans l'ordre alphabétique une liste
de compétiteurs.


**À faire**

De manière similaire, faites le travail nécessaire pour pouvoir trier
une liste de compétiteurs par ordre croissant des performances
réalisées. Les compétiteurs sans résultat sont placés en fin de liste
par ordre alphabétique.


Publication et sauvegarde des résultats
---------------------------------------

### Affichage des résultats

Il est temps de procéder à la publication des résultats.


**À faire**

Réalisez une fonction nommée `print_results` paramétrée par une liste de
candidats qui imprime sur la sortie standard cette liste en précisant
leur prénom, nom, sexe, numéro de dossard et performance.

Par exemple, avec le petit jeu de données, et en supposant que le report
des performances a été effectué et la liste de compétiteurs triée par
ordre de mérite, on obtient un affichage de la forme :

``` {.sourceCode .bash}
Prénom     Nom        Sex  Num.  Performance
Archard    Rivard      M      7  00:46:31
Cheney     Chassé      M      8  00:48:10
Saville    Marier      M      4  00:56:29
Namo       Lereau      M      5  01:06:20
Sidney     Charest     M     10  01:06:38
Sidney     Robert      M      1  01:08:55
Romaine    Hughes      F      6  01:17:08
Vincent    Riquier     M      3  01:21:23
Paien      Gilbert     M      2  
Avelaine   CinqMars    F      9  
```

**À faire**

Produisez l\'affichage des résultats par ordre alphabétique, et par
ordre de mérite.


### Sauvegarde des résultats

Enfin pour la pérennité de ces résultats, il est important de les
sauvegarder dans un fichier.


**À faire**

Réalisez une fonction nommée `save_results` paramétrée par une liste de
candidats et un nom de fichier de sauvegarde, qui crée un fichier au
format CSV contenant

-   les libellés en première ligne

    ``` {.sourceCode .text}   
    Prénom;Nom;Sexe;Num_dossard;Performance
    ```

-   les résultats sur les lignes suivantes

    ``` {.sourceCode .text}   
    Archard;Rivard;M;7;00:46:31
    Cheney;Chassé;M;8;00:48:10
    ...
    ```

**À faire**

Testez votre fonction avec le petit jeu de données puis sauvegardez les résultats de la course.

L'équipe organisatrice de la course du chicon vous remercie
chaleureusement pour votre contribution à son bon déroulement.
