from liste_7776_mots import LISTE_MOTS
import random 

# Ne vérifie pas l'unicité des mots
def verif() :
    return all(isinstance(mot, str) for mot in LISTE_MOTS)\
        and all(all(c.upper() in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' for c in mot) for mot in LISTE_MOTS)\
        and len(set(LISTE_MOTS)) == 7776
        
def premier_2094_dernier() :
    return [str(i) + '-' + LISTE_MOTS[i] for i in (0, 2094, 7775)]

def lmin_lmax():
    lmin = len(LISTE_MOTS[0])
    lmax = lmin
    for mot in LISTE_MOTS:
        long = len(mot)
        if lmin > long:
            lmin = long
        if lmax < long:
            lmax = long
    return lmin, lmax

def print_prem_2095_dernier():
    for k in (0, len(LISTE_MOTS)-1, 2094):
        print(k, ':', LISTE_MOTS[k])
        
        
def en_nombre(lancers):
    '''
    :param lancers: (str) une chaîne décrivant des lancers de dés.
    :return: (int) le nombre correspondant aux lancers de dés définis par lancers
    :CU: lancers ne doit contenir que les chiffres de 1 à 6.
    :Exemples:
    
    >>> en_nombre('11111')
    0
    >>> en_nombre('66666')
    7775
    >>> en_nombre('24521')
    2094
    '''
    res = 0
    for c in lancers:
        res = 6*res + int(c) - 1
    return res

def donne_mot(lancers):
    '''
    :param lancers: (str) une chaîne décrivant des lancers de dés.
    :return: (str) le mot correspondant aux cinq lancers de dés définis par lancers
    :CU: lancers doit être de longueur 5 et ne contenir que les chiffres de 1 à 6.
    :Exemples:
    
    >>> donne_mot('11111')
    'montes'
    >>> donne_mot('66666')
    'tour'
    >>> donne_mot('24521')
    'morigener'
    '''
    return LISTE_MOTS[en_nombre(lancers)]

def genere_phrase_passe(lancers):
    '''
    :param lancers: (str) une chaîne décrivant des lancers de dés.
    :return: (str) la phrase de passe correspondant aux lancers de dés définis par lancers
    :CU: lancers doit être de longueur multiple de 5 et ne contenir que les chiffres de 1 à 6.
    :Exemples:

    >>> genere_phrase_passe('11111')
    'montes'
    >>> genere_phrase_passe('111116666624521')
    'montes-tour-morigener'
    '''
    res = ''
    for k in range(len(lancers)//5):
        res = res + '-' + donne_mot(lancers[5*k:5*(k+1)])
    return res[1:] # on supprime le '-' initial


def est_lettre(c):
    '''
    :param c: (str) un caractère
    :return: (bool) 
       - True si c est une lettre
       - False sinon
    :CU: c doit être une chaîne de longueur 1
    :Exemples:
    
    >>> est_lettre('a')
    True
    >>> est_lettre('B')
    True
    >>> est_lettre('1')
    False
    '''
    return c.upper() in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'

def vrai_ou_faux():
    return random.choice((True, False))

def perturbe_chaine(chaine):
    '''
    :param chaine: (str) une chaîne de caractères
    :return: (str) la chaîne passée en paramètres dans laquelle toutes les lettres ont été changées ou non
            aléatoirement en majuscule ou minuscule
    :CU: aucune
    :Exemples:
    
    >>> perturbe_chaine('oui') in ('OUI', 'OUi', 'OuI', 'Oui', 'oUI', 'oUi', 'ouI', 'oui')
    >>> perturbe_chaine('1a-') in ('1A-', '1a-')
    '''
    res = ''
    for c in chaine:
        if vrai_ou_faux():
            res = res + c.lower()
        else:
            res = res + c.upper()
    return res

def genere_phrase_passe2(lancers):
    '''
    :param lancers: (str) une chaîne décrivant des lancers de dés.
    :return: (str) la phrase de passe correspondant aux lancers de dés définis par lancers,
             les lettres étant mises aléatoirement en majuscule ou minuscule.
    :CU: lancers doit être de longueur multiple de 5 et ne contenir que les chiffres de 1 à 6.
    '''
    return perturbe_chaine(genere_phrase_passe(lancers))


EQUIVALENTS = {
    'a' : '@',
    'b' : '8',
    'e' : '3',
    'i' : '1',
    'o' : '0'
}


def perturbe_chaine2(chaine):
    '''
    :param chaine: (str) une chaîne de caractères
    :return: (str) la chaîne passée en paramètres dans laquelle toutes les lettres ont été changées ou non
            aléatoirement en majuscule ou minuscule et les lettres 
    :CU: aucune
    '''
    res = ''
    for c in chaine:
        if pile_ou_face() == 'P':
            res = res + c.lower()
        elif c in EQUIVALENTS and pile_ou_face() =='P':
            res = res + EQUIVALENTS[c]
        else:
            res = res + c.upper()
    return res

def genere_phrase_passe3(lancers):
    '''
    :param lancers: (str) une chaîne décrivant des lancers de dés.
    :return: (str) la phrase de passe correspondant aux lancers de dés définis par lancers,
             les lettres étant mises aléatoirement en majuscule ou minuscule ou un caractère équivalent.
    :CU: lancers doit être de longueur multiple de 5 et ne contenir que les chiffres de 1 à 6.
    '''
    return perturbe_chaine2(genere_phrase_passe(lancers))
